# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/python-ftp-svn/source/Binary2Ascii.py $
# Last committed:     $Revision: 214405 $; $Author: svcdms $; $Date: 2019-07-12 08:39:56 +0200 (vr, 12 jul 2019) $
#
from dmsLogger.DmsLogger import DmsLogger
from errno import EEXIST
from misc.Utils import Utils
from os import walk, path, makedirs, remove
from re import sub
from svn import local, exception
import getopt
import subprocess
import sys


def create_folder(logger, file):
    directory = path.dirname(file)
    try:
        makedirs(directory)
        logger.info("run_conversion: created folder {}".format(directory))
    except (OSError, FileExistsError) as e:
        if e.errno != EEXIST:
            logger.error("run_conversion: could not create folder {}".format(directory))
            raise


def svn_commit(logger, output_folder, file):
    lc = local.LocalClient(output_folder)

    logger.info("svn_commit: Adding {}".format(file))
    spath = output_folder
    subfolders = sub(output_folder, "", path.dirname(file))

    for subfolder in subfolders.split("\\"):
        spath = spath + "/" + subfolder
        try:
            lc.add(spath)
        except exception.SvnException:
            continue
    try:
        lc.add(file)
    except exception.SvnException:
        return

    lc.cleanup()
    lc.update()
    lc.commit("commit from automatic process")


def run_conversion(settings):
    logger = DmsLogger.set_logger(settings["log_folder"], settings["logger"], settings["logfile"])
    logger.info("Binary2Ascii: $Revision: 214405 $")

    for dirName, subdirList, fileList in walk(settings["import_folder"]):
        logger.info("{}: Process files in folder {} (number of files: {})".format(settings["logger"], dirName, len(fileList)))
        for fname in fileList:
            full_path_in = path.join(dirName, fname)
            if path.getsize(full_path_in) == 0:
                logger.warning("Input file is 0 bytes: {}. File will be removed".format(fname))
                remove(full_path_in)
                continue

            logger.debug("Processing file: {}".format(fname))

            # create output folder if it does not exist
            outfile = sub("BIN$", "asc", fname)
            full_path_out = path.join(dirName, outfile)
            full_path_out = sub(settings["import_folder"], settings["output_folder"], full_path_out)
            create_folder(logger, full_path_out)

            # create log folder if it does not exist
            logfile = settings["logger"] + ".log"
            full_path_log = path.join(dirName, logfile)
            full_path_log = sub(settings["import_folder"], settings["log_folder"], full_path_log)
            create_folder(logger, full_path_log)

            cmd = "{} {} {} {}".format(settings["executable"], full_path_in, full_path_out, full_path_log)
            logger.info("Executing {}".format(cmd))
            subprocess.call(cmd)
            logger.debug("Remove {}".format(full_path_in))
            remove(full_path_in)

            # tijdelijk uit; te veel voor svn; eerst zip nodig
            # svn_commit(logger, settings['output_folder'], full_path_out)

            # deze doen we ook voorlopig niet:
            # create backup folder if it does not exist
            #full_path_backup = sub(settings["output_folder"], settings["import_backup"], full_path_out)
            #create_folder(logger, full_path_backup)
            #if path.isfile(full_path_out):
            #    copy2(full_path_out, full_path_backup)
            #else:
            #    logger.warning("Output file {} not generated".format(full_path_out))


def main(argv):
    # input argument checking
    try:
        opts, args = getopt.getopt(argv, "hs:", ["settings="])
    except getopt.GetoptError:
        print("Binary2Ascii.py -s <settings_file>")
        sys.exit(2)
    for opt, arg in opts:
        if opt == "-h":
            print("Binary2Ascii.py -s <settings_file>")
            sys.exit()
        elif opt in ("-s", "--settings"):
            settings_file = arg

    try:
        settings = Utils.read_bin2ascii_settings(settings_file)
        run_conversion(settings)
    except FileNotFoundError:
        exit(1)


if __name__ == "__main__":
    main(sys.argv[1:])
